@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2006 Frank Denis. All rights reserved.
+ * Copyright (c) 2004-2007 Frank Denis. All rights reserved.
  * 
  * This crucial part of UCARP is derived from the OpenBSD project.
  * Original copyright follows. 
@@ -185,6 +185,7 @@ static void carp_send_ad(struct carp_softc *sc)
     size_t eth_len;
     int advbase;
     int advskew;
+    int rc;
 
 #ifdef DEBUG
     logfile(LOG_DEBUG, "-> carp_send_ad()");
@@ -226,9 +227,8 @@ static void carp_send_ad(struct carp_softc *sc)
     ip.ip_p = IPPROTO_CARP;
     ip.ip_sum = 0;
     
-    memcpy(&ip.ip_src, &srcip, sizeof ip.ip_src);
-    
-    ip.ip_dst.s_addr = INADDR_CARP_GROUP;
+    memcpy(&ip.ip_src, &srcip, sizeof ip.ip_src);    
+    memcpy(&ip.ip_dst.s_addr, inaddr_carp_group, sizeof ip.ip_dst.s_addr);
     
     carp_prepare_ad(&ch, sc);       
     
@@ -260,11 +260,12 @@ static void carp_send_ad(struct carp_softc *sc)
     sum = cksum(ip_ptr, ip_len);
     ip_ptr[offsetof(struct ip, ip_sum)] = (sum >> 8) & 0xff;
     ip_ptr[offsetof(struct ip, ip_sum) + 1] = sum & 0xff;
-        
-    if (write(dev_desc_fd, pkt, eth_len) < 0) {
-#ifdef DEBUG
-        logfile(LOG_WARNING, _("write() has failed"));
-#endif
+
+    do {
+	rc = write(dev_desc_fd, pkt, eth_len);
+    } while (rc < 0 && errno == EINTR);
+    if (rc < 0) {
+        logfile(LOG_WARNING, _("write() has failed: %s"), strerror(errno));
         if (sc->sc_sendad_errors < INT_MAX) {
             sc->sc_sendad_errors++;
         }
@@ -295,6 +296,15 @@ static void carp_send_ad(struct carp_softc *sc)
 #endif
     
     ALLOCA_FREE(pkt);
+    
+    if (sc->sc_delayed_arp > 0)
+	sc->sc_delayed_arp--;
+    if (sc->sc_delayed_arp == 0) {
+	if (sc->sc_state == MASTER) {
+	    gratuitous_arp(dev_desc_fd, 1);
+	}
+	sc->sc_delayed_arp = -1;
+    }    
     if (advbase != 255 || advskew != 255) {
         sc->sc_ad_tmo.tv_sec = now.tv_sec + tv.tv_sec;
         sc->sc_ad_tmo.tv_usec = now.tv_usec + tv.tv_usec;            
@@ -303,7 +313,7 @@ static void carp_send_ad(struct carp_softc *sc)
 }
 
 static void carp_send_ad_all(struct carp_softc *sc) {
-    return carp_send_ad(sc);
+    carp_send_ad(sc);
 }
 
 static void carp_setrun(struct carp_softc *sc, sa_family_t af)
@@ -315,14 +325,8 @@ static void carp_setrun(struct carp_softc *sc, sa_family_t af)
 #endif
     switch (sc->sc_state) {
     case INIT:
-	if (preempt != 0 && carp_suppress_preempt == 0) {
-	    carp_set_state(sc, MASTER);
-	    carp_send_ad(sc);
-	    gratuitous_arp(dev_desc_fd, 1);
-	} else {
-	    carp_set_state(sc, BACKUP);
-	    carp_setrun(sc, 0);	    
-	}
+	carp_set_state(sc, BACKUP);
+	carp_setrun(sc, 0);	    
         break;
     case BACKUP:
         sc->sc_ad_tmo.tv_sec = 0;
@@ -376,6 +380,8 @@ static void carp_master_down(struct carp_softc *sc)
         carp_set_state(sc, MASTER);
         carp_send_ad(sc);
         gratuitous_arp(dev_desc_fd, 0);
+	/* Schedule a delayed ARP request to deal w/ some L3 switches */
+	sc->sc_delayed_arp = 2;
 #ifdef INET6
         carp_send_na(sc);
 #endif /* INET6 */
@@ -635,15 +641,18 @@ char *build_bpf_rule(void)
 
 int docarp(void)
 {
+    char errbuf[PCAP_ERRBUF_SIZE];
     struct bpf_program bpfp;
     struct pollfd pfds[1];
+    struct ifreq iface;
+    int fd;
     int nfds;
-    char errbuf[PCAP_ERRBUF_SIZE];
 
     sc.sc_vhid = vhid;
     sc.sc_advbase = advbase;
     sc.sc_advskew = advskew;
     sc.sc_init_counter = 1;
+    sc.sc_delayed_arp = -1;
 #ifdef INET6
     sc.sc_im6o.im6o_multicast_hlim = CARP_DFLTTL;
 #endif /* INET6 */
@@ -666,7 +675,7 @@ int docarp(void)
 
     if (fill_mac_address() != 0) {
         logfile(LOG_ERR, _("Unable to find MAC address of [%s]"),
-                interface);
+                interface == NULL ? "-" : interface);
         return -1;
     }
     logfile(LOG_INFO, _("Local advertised ethernet address is "
@@ -677,19 +686,19 @@ int docarp(void)
     if ((dev_desc = pcap_open_live(interface, ETHERNET_MTU, 0,
                                    CAPTURE_TIMEOUT, errbuf)) == NULL) {
         logfile(LOG_ERR, _("Unable to open interface [%s]: %s"),
-                interface, errbuf);
+                interface == NULL ? "-" : interface, errbuf);
         return -1;
     }    
     if (pcap_compile(dev_desc, &bpfp, build_bpf_rule(),
                      1, (bpf_u_int32) 0) != 0) {
         logfile(LOG_ERR, _("Unable to compile pcap rule: %s [%s]"),
-                errbuf, interface);
+                errbuf, interface == NULL ? "-" : interface);
         return -1;
     }
     pcap_setfilter(dev_desc, &bpfp);
     dev_desc_fd = pcap_fileno(dev_desc);
     pfds[0].fd = dev_desc_fd;
-    pfds[0].events = POLLIN | POLLERR | POLLHUP | POLLNVAL;
+    pfds[0].events = POLLIN | POLLERR | POLLHUP;
     
     if (shutdown_at_exit != 0) {
         (void) signal(SIGINT, sighandler_exit);
@@ -698,22 +707,52 @@ int docarp(void)
     }
 
     if (gettimeofday(&now, NULL) != 0) {
-	logfile(LOG_WARNING, "initializing now to gettimeofday() failed: %s",
+	logfile(LOG_WARNING, _("initializing now to gettimeofday() failed: %s"),
 		strerror(errno));
     }
     carp_setrun(&sc, 0); 
     
+    if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) == -1) {
+        logfile(LOG_ERR, _("Error opening socket for interface [%s]: %s"),
+		interface == NULL ? "-" : interface, strerror(errno));
+        return -1;
+    }
+#ifdef SIOCGIFFLAGS    
+    if (strlen(interface) >= sizeof iface.ifr_name) {
+	logfile(LOG_ERR, _("Interface name too long"));
+	return -1;
+    }
+    strncpy(iface.ifr_name, interface, sizeof iface.ifr_name);
+#endif    
     for (;;) {
-        nfds = poll(pfds, (nfds_t) 1, sc.sc_advbase * 1000);
-        if (nfds == -1 || 
-            (pfds[0].revents & (POLLERR | POLLHUP | POLLNVAL)) != 0) {
-#ifdef DEBUG
-            logfile(LOG_NOTICE, "poll() error");
+#ifdef SIOCGIFFLAGS
+        if (ioctl(fd, SIOCGIFFLAGS, &iface) != 0) {
+	    break;
+	}
+	if ((iface.ifr_flags & IFF_RUNNING) == 0) {
+            carp_set_state(&sc, BACKUP);
+	    sc.sc_ad_tmo.tv_sec = 0;
+	    sc.sc_ad_tmo.tv_usec = 0;
+	    sc.sc_md_tmo.tv_sec = 0;
+	    sc.sc_md6_tmo.tv_usec = 0;
+	    sleep(SECONDS_TO_WAIT_AFTER_INTERFACE_IS_DOWN);
+	    continue;
+	}
 #endif
+        nfds = poll(pfds, (nfds_t) 1, sc.sc_advbase * 1000);
+        if (nfds == -1) {
+	    if (errno == EINTR) {
+		continue;
+	    }
+	    logfile(LOG_ERR, _("exiting: poll() error: %s"), strerror(errno));
+	}
+	if ((pfds[0].revents & (POLLERR | POLLHUP)) != 0) {
+            logfile(LOG_ERR, _("exiting: pfds[0].revents = %d"),
+		    pfds[0].revents);
             break;
         }
         if (gettimeofday(&now, NULL) != 0) {
-            logfile(LOG_WARNING, "gettimeofday() failed: %s",
+            logfile(LOG_WARNING, _("gettimeofday() failed: %s"),
                     strerror(errno));
             continue;
         }        
