@@ -20,18 +20,26 @@ int spawn_handler(const int dev_desc_fd, const char * const script)
     pid = fork();
     if (pid == (pid_t) 0) {
         (void) close(dev_desc_fd);
-        execl(script, script, interface, (char *) NULL);
+        execl(script, script, interface, vaddr_arg, xparam, (char *) NULL);
+	logfile(LOG_ERR, _("Unable to exec %s %s: %s"), script, interface,
+		strerror(errno));
         _exit(EXIT_FAILURE);
     } else if (pid != (pid_t) -1) {
         logfile(LOG_WARNING, _("Spawning [%s %s]"), script, interface);
 #ifdef HAVE_WAITPID
-        (void) waitpid(pid, NULL, 0);
+	{
+	    while (waitpid(pid, NULL, 0) == (pid_t) -1 && errno == EINTR);
+	}
 #else
         {
             pid_t foundpid;
             
-            while ((foundpid = wait3(NULL, 0, NULL)) != (pid_t) -1 &&
-                   foundpid != pid);
+	    do {
+		foundpid = wait3(NULL, 0, NULL);
+		if (foundpid == (pid_t) -1 && errno == EINTR) {
+		    continue;
+		}
+	    } while (foundpid != (pid_t) -1 && foundpid != pid);
         }
 #endif
     } else {
