@@ -1,7 +1,7 @@
 #ifndef __CARP_P_H__
 #define __CARP_P_H__ 1
 
-static const char *GETOPT_OPTIONS = "i:s:v:p:Pa:hb:k:u:d:r:zf:B";
+static const char *GETOPT_OPTIONS = "i:s:v:p:Pa:hb:k:m:nu:d:r:zf:B";
 
 static struct option long_options[] = {
     { "interface", 1, NULL, 'i' },
@@ -20,6 +20,8 @@ static struct option long_options[] = {
     { "shutdown", 0, NULL, 'z' },
     { "facility", 1, NULL, 'f' },
     { "daemonize", 0, NULL, 'B' },
+    { "passfile", 1, NULL, 'o' },
+    { "xparam", 1, NULL, 'm' },
     { NULL, 0, NULL, 0 }
 };
 
